@@ -1155,6 +1155,7 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 	int i;
 	int indent = 0;
 	int last_indent = 0;
+	gint pixel_size = html_painter_get_pixel_size (painter);
 
 	flow = HTML_CLUEFLOW (self);
 
@@ -1164,8 +1165,8 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 		html_painter_set_pen (painter, &html_colorset_get_color_allocated (painter, HTMLLinkColor)->color);
 		if (is_cite (flow, i)) {
 			if (!HTML_IS_PLAIN_PAINTER (painter)) {
-				area.x0 = self->x + indent - 5;
-				area.x1 = area.x0 + 2;
+				area.x0 = self->x + indent - 5 * pixel_size;
+				area.x1 = area.x0 + 2 * pixel_size;
 				area.y0 = self->y - self->ascent;
 				area.y1 = self->y + self->descent;
 				
