@@ -27,6 +27,7 @@
 
 #define HTML_LINK_TEXT(x) ((HTMLLinkText *)(x))
 #define HTML_LINK_TEXT_CLASS(x) ((HTMLLinkTextClass *)(x))
+#define HTML_IS_LINK_TEXT(x) (HTML_CHECK_TYPE ((x), HTML_TYPE_LINKTEXT))
 
 struct _HTMLLinkText {
 	HTMLText text;
@@ -64,8 +65,10 @@ HTMLObject *html_link_text_new_with_len  (const gchar       *text,
 					  HTMLColor         *color,
 					  const gchar       *url,
 					  const gchar       *target);
+HTMLObject *html_link_text_to_text       (HTMLLinkText      *link,
+					  HTMLEngine        *engine);
 void        html_link_text_set_url       (HTMLLinkText      *link,
-					  const gchar       *url);
-void        html_link_text_to_text       (HTMLLinkText      *link,
-					  HTMLEngine        *e);
+					  gchar             *url,
+					  gchar             *target);
+					  
 #endif /* _HTMLLINKTEXT_H_ */
