@@ -175,8 +175,8 @@ op_copy (HTMLObject *self, HTMLObject *parent, HTMLEngine *e, GList *from, GList
 	t  = HTML_TABLE (self);
 	nt = g_new0 (HTMLTable, 1);
 
-	start = HTML_TABLE_CELL (from ? from->data : html_object_head (self));
-	end   = HTML_TABLE_CELL (to   ? to->data   : html_object_tail (self));
+	start = HTML_TABLE_CELL ((from && from->next) ? from->data : html_object_head (self));
+	end   = HTML_TABLE_CELL ((to && to->next)     ? to->data   : html_object_tail (self));
 	rows  = end->row - start->row + 1;
 	cols  = end->row == start->row ? end->col - start->col + 1 : t->totalCols;
 
@@ -1101,6 +1101,9 @@ get_bounds (HTMLTable *table, gint x, gint y, gint width, gint height, gint *sc,
 {
 	g_return_if_fail (table->rowHeights);
 	g_return_if_fail (table->columnOpt);
+	g_return_if_fail (table->rowHeights->data);
+	g_return_if_fail (table->columnOpt->data);
+
 
 	*sr = to_index (bin_search_index (table->rowHeights, 0, table->totalRows, y), 0, table->totalRows - 1);
 	if (y < ROW_HEIGHT (table, *sr) && (*sr) > 0)
@@ -1641,7 +1644,7 @@ html_table_set_max_width (HTMLObject *o,
 	pixel_size   = html_painter_get_pixel_size (painter);
 	o->max_width = MAX (html_object_calc_min_width (o, painter), max_width);
 	max_width    = o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH
-		? max_width = pixel_size * table->specified_width
+		? pixel_size * table->specified_width
 		: (o->percent
 		   ? ((gdouble) MIN (100, o->percent) / 100 * max_width)
 		   : MIN (html_object_calc_preferred_width (HTML_OBJECT (table), painter), max_width));
@@ -1730,13 +1733,37 @@ check_point (HTMLObject *self,
 	HTMLTableCell *cell;
 	HTMLObject *obj;
 	HTMLTable *table;
-	gint r, c, start_row, end_row, start_col, end_col;
+	gint r, c, start_row, end_row, start_col, end_col, hsb, hsa, tbc;
 
 	if (x < self->x || x >= self->x + self->width
 	    || y >= self->y + self->descent || y < self->y - self->ascent)
 		return NULL;
 
 	table = HTML_TABLE (self);
+	hsb = table->spacing >> 1;
+	hsa = hsb + (table->spacing & 1);
+	tbc = table->border ? 1 : 0;
+
+	if (for_cursor) {
+		/* table boundaries */
+		if (x == self->x || x == self->x + self->width - 1) {
+			if (offset_return)
+				*offset_return = x == self->x ? 0 : 1;
+			return self;
+		}
+
+		/* border */
+		if (x < self->x + table->border + hsb || y < self->y - self->ascent + table->border + hsb) {
+			if (offset_return)
+				*offset_return = 0;
+			return self;
+		}
+		if (x > self->x + self->width - table->border - hsa || y > self->y + self->descent - table->border - hsa) {
+			if (offset_return)
+				*offset_return = 1;
+			return self;
+		}
+	}
 
 	x -= self->x;
 	y -= self->y - self->ascent;
@@ -1744,6 +1771,9 @@ check_point (HTMLObject *self,
 	get_bounds (table, x, y, 0, 0, &start_col, &end_col, &start_row, &end_row);
 	for (r = start_row; r <= end_row; r++) {
 		for (c = 0; c < table->totalCols; c++) {
+			HTMLObject *co;
+			gint cx, cy;
+
 			cell = table->cells[r][c];
 			if (cell == NULL)
 				continue;
@@ -1753,48 +1783,25 @@ check_point (HTMLObject *self,
 			if (r < end_row - 1 && table->cells[r+1][c] == cell)
 				continue;
 
-			obj = html_object_check_point (HTML_OBJECT (cell), painter, x, y, offset_return, for_cursor);
+			/* correct to include cell spacing */
+			co = HTML_OBJECT (cell);
+			cx = x;
+			cy = y;
+			if (x < co->x && x >= co->x - hsa - tbc)
+				cx = co->x;
+			if (x >= co->x + co->width && x < co->x + co->width + hsb + tbc)
+				cx = co->x + co->width - 1;
+			if (y < co->y - co->ascent && y >= co->y - co->ascent - hsa - tbc)
+				cy = co->y - co->ascent;
+			if (y >= co->y + co->descent && y < co->y + co->descent + hsb + tbc)
+				cy = co->y + co->descent - 1;
+
+			obj = html_object_check_point (HTML_OBJECT (cell), painter, cx, cy, offset_return, for_cursor);
 			if (obj != NULL)
 				return obj;
 		}
 	}
 
-	if (for_cursor) {
-
-		/* check before */
-		for (c=0; c<table->totalCols; c++)
-			if (table->cells [start_row][c])
-				break;
-		if (c < table->totalCols && table->cells [start_row][c]) {
-			cell = table->cells [start_row][c];
-			if (x < HTML_OBJECT (cell)->x || y < HTML_OBJECT (cell)->y - HTML_OBJECT (cell)->ascent) {
-				obj = html_object_check_point (HTML_OBJECT (cell), painter,
-							       HTML_OBJECT (cell)->x,
-							       HTML_OBJECT (cell)->y - HTML_OBJECT (cell)->ascent,
-							       offset_return, for_cursor);
-				if (obj)
-					return obj;
-			}
-		}
-
-		/* check after */
-		for (c=table->totalCols - 1; c >= 0; c--)
-			if (table->cells [start_row][c])
-				break;
-		if (c >=0 && table->cells [start_row][c]) {
-			cell = table->cells [start_row][c];
-			if (x > HTML_OBJECT (cell)->x + HTML_OBJECT (cell)->width - 1
-			    || y > HTML_OBJECT (cell)->y + HTML_OBJECT (cell)->descent - 1) {
-				obj = html_object_check_point (HTML_OBJECT (cell), painter,
-							       HTML_OBJECT (cell)->x + HTML_OBJECT (cell)->width - 1,
-							       HTML_OBJECT (cell)->y + HTML_OBJECT (cell)->descent - 1,
-							       offset_return, for_cursor);
-				if (obj)
-					return obj;
-			}
-		}
-	}
-
 	return NULL;
 }
 
@@ -2290,17 +2297,21 @@ html_table_end_row (HTMLTable *table)
 	table->row++;
 }
 
-void
+gint
 html_table_end_table (HTMLTable *table)
 {
-	gint r, c;
+	gint r, c, cells = 0;
 
 	for (r = 0; r < table->totalRows; r ++)
 		for (c = 0; c < table->totalCols; c ++)
-			if (table->cells [r][c] && HTML_CLUE (table->cells [r][c])->head == NULL) {
-				HTMLTableCell *cell = table->cells [r][c];
+			if (table->cells [r][c]) {
+				if (HTML_CLUE (table->cells [r][c])->head == NULL) {
+					HTMLTableCell *cell = table->cells [r][c];
 
-				remove_cell (table, cell);
-				html_object_destroy (HTML_OBJECT (cell));
+					remove_cell (table, cell);
+					html_object_destroy (HTML_OBJECT (cell));
+				} else
+					cells ++;
 			}
+	return cells;
 }
