@@ -32,6 +32,8 @@
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
 #include "htmlpainter.h"
+#include "htmlplainpainter.h"
+#include "htmlgdkpainter.h"
 
 
 /* #define HTML_TEXT_SLAVE_DEBUG */
@@ -39,10 +41,103 @@
 HTMLTextSlaveClass html_text_slave_class;
 static HTMLObjectClass *parent_class = NULL;
 
+#ifndef GAL_UTF8_NOT_SLOW
+/*
+ * NOTE:
+ * currently the gal utf8 functions check the entire string you pass to them for
+ * invalid utf8, the expense of those calls is massive inside gtkhtml when a paragraph
+ * is long.  Until gal turns off the checks we will use versions without checks.
+ * And yes, it really is worth it.
+ *   
+ * --Larry
+ */
+gint h_utf8_pointer_to_offset (const gchar *str, const gchar *pos);
+gint h_utf8_strlen (const gchar *p, gint max);
+gchar *h_utf8_offset_to_pointer  (const gchar *str, gint         offset);
+
+gint
+h_utf8_pointer_to_offset (const gchar *str,
+			  const gchar *pos)
+{
+  const gchar *s = str;
+  gint offset = 0;
+
+  while (s < pos)
+    {
+      s = g_utf8_next_char (s);
+      offset++;
+    }
+
+  return offset;
+}
+
+gint
+h_utf8_strlen (const gchar *p, gint max)
+{
+  int len = 0;
+  const gchar *start = p;
+
+  if (max < 0)
+    {
+      while (*p)
+        {
+          p = g_utf8_next_char (p);
+          ++len;
+        }
+    }
+  else
+    {
+      if (max == 0 || !*p)
+        return 0;
+      
+      p = g_utf8_next_char (p);          
+
+      while (p - start < max && *p)
+        {
+          ++len;
+          p = g_utf8_next_char (p);          
+        }
+
+      /* only do the last len increment if we got a complete
+       * char (don't count partial chars)
+       */
+      if (p - start == max)
+        ++len;
+    }
+
+  return len;
+}
+
+gchar *
+h_utf8_offset_to_pointer  (const gchar *str,
+			   gint         offset)
+{
+  const gchar *s = str;
+
+  while (offset--)
+    s = g_utf8_next_char (s);
+  
+  return (gchar *)s;
+}
+
+#define g_utf8_strlen h_utf8_strlen
+#define g_utf8_offset_to_pointer h_utf8_offset_to_pointer
+#define g_utf8_pointer_to_offset h_utf8_pointer_to_offset
+#endif
+
 
+char *
+html_text_slave_get_text (HTMLTextSlave *slave)
+{
+	if (!slave->charStart)
+		slave->charStart = html_text_get_text (slave->owner, slave->posStart);
+
+	return slave->charStart;
+}
+
 /* Split this TextSlave at the specified offset.  */
 static void
-split (HTMLTextSlave *slave, guint offset, guint start_word)
+split (HTMLTextSlave *slave, guint offset, guint start_word, char *start_pointer)
 {
 	HTMLObject *obj;
 	HTMLObject *new;
@@ -55,6 +150,9 @@ split (HTMLTextSlave *slave, guint offset, guint start_word)
 	new = html_text_slave_new (slave->owner,
 				   slave->posStart + offset,
 				   slave->posLen - offset, start_word);
+
+	HTML_TEXT_SLAVE (new)->charStart = start_pointer;
+
 	html_clue_append_after (HTML_CLUE (obj->parent), new, obj);
 
 	slave->posLen = offset;
@@ -75,7 +173,7 @@ copy (HTMLObject *self,
 	HTML_TEXT_SLAVE (dest)->posLen = HTML_TEXT_SLAVE (self)->posLen;
 }
 
-inline static guint
+static guint
 get_words_width (HTMLText *text, HTMLPainter *p, guint start_word, guint words)
 {
 	return text->word_width [start_word + words - 1]
@@ -89,6 +187,7 @@ calc_width (HTMLTextSlave *slave, HTMLPainter *painter)
 	HTMLText *text = slave->owner;
 	HTMLObject *next, *prev;
 
+	html_text_request_word_width (text, painter);
 	if (slave->posStart == 0 && slave->posLen == text->text_len)
 		return text->word_width [text->words - 1];
 
@@ -100,10 +199,9 @@ calc_width (HTMLTextSlave *slave, HTMLPainter *painter)
 		&& slave->start_word == HTML_TEXT_SLAVE (next)->start_word)) {
 		gint line_offset = -1;
 
-		return html_painter_calc_text_width (painter, html_text_get_text (text, slave->posStart), slave->posLen,
-										  &line_offset,
-						     html_text_get_font_style (text),
-										  text->face);
+		return html_painter_calc_text_width (painter, html_text_slave_get_text (slave), slave->posLen,
+						     &line_offset, html_text_get_font_style (text),
+						     text->face);
 	} else
 		return get_words_width (text, painter, slave->start_word,
 					(next && HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE
@@ -119,7 +217,7 @@ get_offset_for_bounded_width (HTMLTextSlave *slave, HTMLPainter *painter, gint *
 	gint len, width;
 	gint line_offset = -1;
 	gchar *sep, *str;
-	char *buffer = html_text_get_text (text, slave->posStart);
+	char *buffer = html_text_slave_get_text (slave);
 
 	len = (lower + upper) / 2;
 	width = html_painter_calc_text_width (painter, buffer, len, &line_offset,
@@ -164,18 +262,18 @@ slave_split_if_too_long (HTMLTextSlave *slave, HTMLPainter *painter, gint *width
 
 		pos = get_offset_for_bounded_width (slave, painter, &words, MAX_WIDGET_WIDTH - x);
 		if (pos > 0 && pos < slave->posLen) {
-			split (slave, pos, slave->start_word + words);
+			split (slave, pos, slave->start_word + words, NULL);
 			*width = MAX (1, calc_width (slave, painter));
 		}
 	}
 }
 
 static gboolean
-calc_size (HTMLObject *self,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *self, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLText *owner;
 	HTMLTextSlave *slave;
+	HTMLObject *next;
 	GtkHTMLFontStyle font_style;
 	gint new_ascent, new_descent, new_width;
 	gboolean changed;
@@ -201,7 +299,12 @@ calc_size (HTMLObject *self,
 	}
 
 	new_width = MAX (1, calc_width (slave, painter));
-	if (new_width > HTML_OBJECT (owner)->max_width)
+
+	next = HTML_OBJECT (slave)->next;
+	if ((slave->start_word == owner->words - 1
+	     && (!next || (HTML_IS_TEXT_SLAVE (next) && HTML_TEXT_SLAVE (next)->start_word == slave->start_word + 1)))
+	    && (HTML_IS_PLAIN_PAINTER (painter) || HTML_IS_GDK_PAINTER (painter))
+	    && new_width > HTML_OBJECT (owner)->max_width)
 		slave_split_if_too_long (slave, painter, &new_width);
 
 	changed = FALSE;
@@ -269,7 +372,7 @@ html_text_slave_get_line_offset (HTMLTextSlave *slave, gint line_offset, gint of
 			prev = html_object_prev (HTML_OBJECT (slave)->parent, HTML_OBJECT (prev));
 
 		bol = HTML_TEXT_SLAVE (prev->next);
-		return html_text_text_line_length (html_text_get_text (bol->owner, bol->posStart),
+		return html_text_text_line_length (html_text_slave_get_text (bol),
 						   0, offset - bol->posStart);
 	} else
 		return line_offset + html_text_text_line_length (slave->owner->text, line_offset, offset);
@@ -294,11 +397,11 @@ get_next_nb_width (HTMLTextSlave *slave, HTMLPainter *painter)
 }
 
 static gboolean
-could_remove_leading_space (HTMLTextSlave *slave, gboolean firstRun)
+could_remove_leading_space (HTMLTextSlave *slave, gboolean lineBegin)
 {
 	HTMLObject *o = HTML_OBJECT (slave->owner);
 
-	if (firstRun && (HTML_OBJECT (slave)->prev != o || o->prev))
+	if (lineBegin && (HTML_OBJECT (slave)->prev != o || o->prev))
 		return TRUE;
 
 	if (!o->prev)
@@ -310,12 +413,46 @@ could_remove_leading_space (HTMLTextSlave *slave, gboolean firstRun)
 	return o->prev ? FALSE : TRUE;
 }
 
+inline gint
+html_text_slave_nb_width (HTMLTextSlave *slave, HTMLPainter *painter, gint words)
+{
+	return get_words_width (slave->owner, painter, slave->start_word, words)
+		+ (slave->start_word + words == slave->owner->words ? get_next_nb_width (slave, painter) : 0);
+}
+
+inline gchar *
+html_text_slave_remove_leading_space (HTMLTextSlave *slave, HTMLPainter *painter, gboolean lineBegin)
+{
+	gchar *begin;
+
+	html_text_request_word_width (slave->owner, painter);
+
+	begin = html_text_slave_get_text (slave);
+	if (*begin == ' ' && could_remove_leading_space (slave, lineBegin)) {
+		if (slave->posStart == 0)
+			slave->start_word ++;
+		begin = g_utf8_next_char (begin);
+		slave->charStart = begin;
+		slave->posStart ++;
+		slave->posLen --;
+	}
+
+	return begin;
+}
+
+gint
+html_text_slave_get_nb_width (HTMLTextSlave *slave, HTMLPainter *painter, gboolean lineBegin)
+{
+	html_text_slave_remove_leading_space (slave, painter, lineBegin);
+	if (slave->owner->words - slave->start_word > 1)
+		return html_text_slave_nb_width (slave, painter, 1);
+
+	return html_object_calc_min_width (HTML_OBJECT (slave), painter);
+}
+
 static HTMLFitType
-fit_line (HTMLObject *o,
-	  HTMLPainter *painter,
-	  gboolean startOfLine,
-	  gboolean firstRun,
-	  gint widthLeft)
+hts_fit_line (HTMLObject *o, HTMLPainter *painter,
+	      gboolean lineBegin, gboolean firstRun, gboolean next_to_floating, gint widthLeft)
 {
 	HTMLFitType rv = HTML_FIT_PARTIAL;
 	HTMLTextSlave *slave;
@@ -330,28 +467,11 @@ fit_line (HTMLObject *o,
 	slave = HTML_TEXT_SLAVE (o);
 	text  = HTML_TEXT (slave->owner);
 
-	html_text_request_word_width (text, painter);
+	begin = html_text_slave_remove_leading_space (slave, painter, lineBegin);
 
-	if (html_text_get_char (text, slave->posStart) == ' ') {
-		if (could_remove_leading_space (slave, firstRun)) {
-			if (slave->posStart == 0)
-				slave->start_word ++;
-			slave->posStart ++;
-			slave->posLen --;
-		} /* else {
-			if (slave->posStart == 0)
-				words ++;
-			else
-				add_width = text->space_width;
-			pos ++;
-			} */
-	}
-
-	sep = begin = html_text_get_text (text, slave->posStart);
-
-	while (sep
-	       && widthLeft >= get_words_width (text, painter, slave->start_word, words + 1)
-	       + (slave->start_word + words + 1 == text->words ? get_next_nb_width (slave, painter) : 0)) {
+	sep = begin;
+	while (sep && *sep
+	       && widthLeft >= html_text_slave_nb_width (slave, painter, words + 1)) {
 		words ++;
 		lsep   = sep;
 		sep    = strchr (lsep + (words > 1 ? 1 : 0), ' ');
@@ -363,25 +483,37 @@ fit_line (HTMLObject *o,
 	if (words + slave->start_word == text->words)
 		rv = HTML_FIT_COMPLETE;
 	else if (words == 0 || get_words_width (text, painter, slave->start_word, words) == 0) {
-		if (!firstRun)
-			rv = HTML_FIT_NONE;
-		else if (slave->start_word + 1 == text->words)
-			rv = HTML_FIT_COMPLETE;
+		if (!firstRun) {
+			if (slave->posStart == 0 && text->text [0] != ' ' && HTML_OBJECT (text)->prev) {
+				HTMLObject *prev = HTML_OBJECT (text)->prev;
+				if (HTML_IS_TEXT_SLAVE (prev) && HTML_TEXT_SLAVE (prev)->posLen
+				    && HTML_TEXT_SLAVE (prev)->owner->text [strlen (HTML_TEXT_SLAVE (prev)->owner->text) - 1]
+				    != ' ')
+					rv = slave->start_word + 1 == text->words ? HTML_FIT_COMPLETE : HTML_FIT_PARTIAL;
+				else
+					rv = HTML_FIT_NONE;
+			} else
+				rv = HTML_FIT_NONE;
+		} else if (slave->start_word + 1 == text->words)
+			rv = next_to_floating ? HTML_FIT_NONE : HTML_FIT_COMPLETE;
 		else {
 			words ++;
-			sep    = strchr (sep + (words > 1 ? 0 : 1), ' ');
+
+			if (sep)
+				sep    = strchr (sep + (words > 1 ? 0 : 1), ' ');
+
 			pos    = sep ? g_utf8_pointer_to_offset (begin, sep) : g_utf8_strlen (begin, -1);
 		}
 	}
 
-	if (rv == HTML_FIT_PARTIAL) {
-		if (pos < slave->posLen)
-			split (slave, pos, slave->start_word + words);
+	if (rv == HTML_FIT_PARTIAL)
+		if (pos < slave->posLen) {
+			split (slave, pos, slave->start_word + words, sep);
 		o->width = get_words_width (text, painter, slave->start_word, words);
 	}
 
 #ifdef HTML_TEXT_SLAVE_DEBUG
-	debug_print (rv, html_text_get_text (text, slave->posStart), slave->posLen);
+	debug_print (rv, html_text_slave_get_text (slave), slave->posLen);
 #endif
 
 	return rv;	
@@ -426,26 +558,35 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty, gint
 	HTMLObject *obj = HTML_OBJECT (slave);
 	SpellError *se;
 	guint ma, mi;
+	gint x_off = 0;
+	gint last_off = 0;
+	gchar *text = html_text_slave_get_text (slave);
 
 	while (cur) {
+
 		se = (SpellError *) cur->data;
 		ma = MAX (se->off, slave->posStart);
 		mi = MIN (se->off + se->len, slave->posStart + slave->posLen);
 		if (ma < mi) {
 			guint off = ma - slave->posStart;
 			guint len = mi - ma;
-			gint x_off;
+			gint lo;
 
 			html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLSpellErrorColor)->color);
 			/* printf ("spell error: %s\n", html_text_get_text (slave->owner, off)); */
-			x_off = html_painter_calc_text_width (p, html_text_get_text (HTML_TEXT (slave->owner),
-										     slave->posStart), off, &line_offset,
-							      p->font_style,
-							      p->font_face);
-			html_painter_draw_spell_error (p, obj->x + tx + x_off,
-						       obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-						       html_text_get_text (HTML_TEXT (slave->owner), slave->posStart + off),
-						       len);
+			lo = line_offset;
+			
+			x_off += html_painter_calc_text_width (p, text,
+							       off - last_off, &line_offset,
+							       p->font_style,
+							       p->font_face);
+			text = g_utf8_offset_to_pointer (text, off - last_off);
+			x_off += html_painter_draw_spell_error (p, obj->x + tx + x_off,
+								obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
+								text, len);
+			last_off = off + len;
+			line_offset += len;
+			text = g_utf8_offset_to_pointer (text, len);
 		}
 		if (se->off > slave->posStart + slave->posLen)
 			break;
@@ -471,7 +612,7 @@ draw_normal (HTMLTextSlave *self,
 	html_painter_set_pen (p, &HTML_TEXT (self->owner)->color->color);
 	html_painter_draw_text (p,
 				obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (self->owner), p),
-				html_text_get_text (HTML_TEXT (self->owner), self->posStart),
+				html_text_slave_get_text (self),
 				self->posLen,
 				html_text_slave_get_line_offset (self, line_offset, self->posStart, p));
 }
@@ -489,6 +630,9 @@ draw_highlighted (HTMLTextSlave *slave,
 	guint start, end, len;
 	gint offset_width, text_width, lo, lo_start, lo_sel;
 	const gchar *text;
+	
+	char *slave_begin;
+	char *highlight_begin;
 
 	obj = HTML_OBJECT (slave);
 	owner = HTML_TEXT (slave->owner);
@@ -503,13 +647,16 @@ draw_highlighted (HTMLTextSlave *slave,
 		end = slave->posStart + slave->posLen;
 	len = end - start;
 
+	slave_begin = html_text_slave_get_text (slave);
+	highlight_begin = g_utf8_offset_to_pointer (slave_begin, start - slave->posStart);
+
 	lo_start = lo = html_text_slave_get_line_offset (slave, line_offset, slave->posStart, p);
-	offset_width = html_painter_calc_text_width (p, g_utf8_offset_to_pointer (text, slave->posStart),
+	offset_width = html_painter_calc_text_width (p, slave_begin,
 						     start - slave->posStart,
 						     &lo,
 						     font_style, HTML_TEXT (owner)->face);
 	lo_sel = lo;
-	text_width = html_painter_calc_text_width (p, g_utf8_offset_to_pointer (text, start),
+	text_width = html_painter_calc_text_width (p, highlight_begin,
 						   len, &lo,
 						   font_style, HTML_TEXT (owner)->face);
 	/* printf ("s: %d l: %d - %d %d\n", start, len, offset_width, text_width); */
@@ -519,18 +666,20 @@ draw_highlighted (HTMLTextSlave *slave,
 
 	/* Draw the highlighted part with a highlight background.  */
 
-	html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLHighlightColor)->color);
+	html_color_alloc (HTML_TEXT (owner)->color, p);
+	html_painter_set_pen (p, &html_colorset_get_color_allocated
+			      (p, p->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color);
 	html_painter_fill_rect (p, obj->x + tx + offset_width, obj->y + ty - obj->ascent,
 				text_width, obj->ascent + obj->descent);
-	html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLHighlightTextColor)->color);
+	html_painter_set_pen (p, &html_colorset_get_color_allocated
+			      (p, p->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color);
 	html_painter_draw_text (p, obj->x + tx + offset_width, 
 				obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-				g_utf8_offset_to_pointer (text, start), len,
+				highlight_begin, len,
 				lo_sel);
 
 	/* Draw the non-highlighted part.  */
 
-	html_color_alloc (HTML_TEXT (owner)->color, p);
 	html_painter_set_pen (p, &HTML_TEXT (owner)->color->color);
 
 	/* 1. Draw the leftmost non-highlighted part, if any.  */
@@ -538,7 +687,7 @@ draw_highlighted (HTMLTextSlave *slave,
 	if (start > slave->posStart)
 		html_painter_draw_text (p,
 					obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-					g_utf8_offset_to_pointer (text, slave->posStart),
+					slave_begin,
 					start - slave->posStart,
 					lo_start);
 
@@ -548,7 +697,7 @@ draw_highlighted (HTMLTextSlave *slave,
 		html_painter_draw_text (p,
 					obj->x + tx + offset_width + text_width,
 					obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-					g_utf8_offset_to_pointer (text, end),
+					g_utf8_offset_to_pointer (highlight_begin, end - start),
 					slave->posStart + slave->posLen - end,
 					lo);
 }
@@ -568,6 +717,8 @@ draw (HTMLObject *o,
 	ArtIRect paint;
 	gint line_offset;
 
+	/* printf ("slave draw %p\n", o); */
+
 	html_object_calc_intersection (o, &paint, x, y, width, height);
 	if (art_irect_empty (&paint))
 		return;
@@ -586,7 +737,8 @@ draw (HTMLObject *o,
 		draw_highlighted (textslave, p, font_style, x, y, width, height, tx, ty, line_offset);
 	}
 
-	draw_spell_errors (textslave, p, tx ,ty, line_offset);
+	if (HTML_TEXT (textslave->owner)->spell_errors)
+		draw_spell_errors (textslave, p, tx ,ty, line_offset);
 }
 
 static gint
@@ -619,7 +771,9 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 	GtkHTMLFontStyle font_style;
 	guint width, prev_width;
 	guint i;
-	gint line_offset, lo;
+	gint line_offset, lo, word, ww;
+	gchar *space;
+	gchar *text;
 
 	g_return_val_if_fail (slave != NULL, 0);
 
@@ -630,10 +784,29 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 
 	prev_width  = 0;
 	line_offset = html_text_slave_get_line_offset (slave, html_text_get_line_offset (slave->owner, painter), 0, painter);
-	for (i = 1; i <= slave->posLen; i++) {
+
+	i = 1;
+	word = 1;
+	text = html_text_slave_get_text (slave);
+	space = strchr (text, ' ');
+	if (space && g_utf8_pointer_to_offset (text, space) <= slave->posLen) {
+		html_text_request_word_width (owner, painter);
+
+		ww = get_words_width (owner, painter, slave->start_word, word);
+		while (ww < x && space) {
+			i = g_utf8_pointer_to_offset (text, space);
+			space = strchr (space + 1, ' ');
+			if (space && g_utf8_pointer_to_offset (text, space) >= slave->posLen) {
+				break;
+			}
+			word ++;
+			ww = get_words_width (owner, painter, slave->start_word, word);
+		}
+	}
+
+	for ( ; i <= slave->posLen; i++) {
 		lo = line_offset;
-		width = html_painter_calc_text_width (painter,
-						      html_text_get_text (owner, slave->posStart),
+		width = html_painter_calc_text_width (painter, text, 
 						      i, &lo, font_style, owner->face);
 
 		if ((width + prev_width) / 2 >= x)
@@ -668,7 +841,6 @@ check_point (HTMLObject *self,
 	return NULL;
 }
 
-
 void
 html_text_slave_type_init (void)
 {
@@ -690,7 +862,7 @@ html_text_slave_class_init (HTMLTextSlaveClass *klass,
 	object_class->copy = copy;
 	object_class->draw = draw;
 	object_class->calc_size = calc_size;
-	object_class->fit_line = fit_line;
+	object_class->fit_line = hts_fit_line;
 	object_class->calc_min_width = calc_min_width;
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->get_url = get_url;
@@ -723,7 +895,7 @@ html_text_slave_init (HTMLTextSlave *slave,
 	slave->posLen     = posLen;
 	slave->start_word = start_word;
 	slave->owner      = owner;
-
+	slave->charStart  = NULL;
 	/* text slaves has always min_width 0 */
 	object->min_width = 0;
 	object->change   &= ~HTML_CHANGE_MIN_WIDTH;
