@@ -520,6 +520,8 @@ on_set_base (GtkHTML *html, const gchar *url, gpointer data)
 	if (baseURL)
 		html_url_destroy (baseURL);
 
+	if (html)
+		gtk_html_set_base (html, url);
 	baseURL = html_url_new (url);
 }
 
@@ -638,7 +640,7 @@ url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer d
 
 	full_url = parse_href (url);
 
-	if (!strncmp (full_url, "http", 4)) {
+	if (full_url && !strncmp (full_url, "http", 4)) {
 		SoupContext *ctx;
 		SoupMessage *msg;
 
@@ -646,7 +648,7 @@ url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer d
 		msg = soup_message_new (ctx, SOUP_METHOD_GET);
 
 		soup_message_queue (msg, got_data, handle);
-	} else if (!strncmp (full_url, "file:", 5)) {
+	} else if (full_url && !strncmp (full_url, "file:", 5)) {
 		struct stat st;
 		char *buf;
 		int fd, nread, total;
@@ -809,11 +811,12 @@ goto_url(const char *url, int back_or_forward)
 
 	html_stream_handle = gtk_html_begin_content (html, "text/html; charset=utf-8");
 
-	/* Yuck yuck yuck.  Well this code is butt-ugly already anyway.  */
-	url_requested (html, url, html_stream_handle, NULL);
+	/* Yuck yuck yuck.  Well this code is butt-ugly already
+	anyway.  */
 
 	full_url = parse_href (url);
 	on_set_base (NULL, full_url, NULL);
+	url_requested (html, url, html_stream_handle, NULL);
 
 	if(!back_or_forward) {
 		if(go_position) {
